﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using ProjectBoat_bae.DrawningObjects;
using ProjectBoat_bae.Generics;
using ProjectBoat_bae.MovementStrategy;

namespace ProjectBoat_bae
{
    public partial class FormBoatCollection : Form
    {
        // Набор объектов
        private readonly BoatsGenericStorage _storage;

        public FormBoatCollection()
        {
            InitializeComponent();
            _storage = new BoatsGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }

        // Заполнение listBoxObjects
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index
            >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }

        // Добавление набора в коллекцию
        private void ButtonAddObject_Click_1(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
        }

        // Удаление набора
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект {listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo,
        MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxStorages.SelectedItem.ToString()
                ?? string.Empty);
                ReloadObjects();
            }
        }

        // Добавление объекта в набор
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1) return;
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null) return;

            FormBoatConfig form = new FormBoatConfig();

            form.AddEvent(AddBoat);
            form.Show();
        }

        private void AddBoat(Drawningboat boat)
        {
            boat._pictureWidth = pictureBoxCollection.Width;
            boat._pictureHeight = pictureBoxCollection.Height;
            if (listBoxStorages.SelectedIndex == -1) return;
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null) return;

            if (obj + boat)
            {
                MessageBox.Show("Объект добавлен");
                pictureBoxCollection.Image = obj.ShowBoats();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }

        //удаление объекта
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = 0;
            if (textBoxBoat != null)
                pos = Convert.ToInt32(textBoxBoat.Text);
            if (obj - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowBoats();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        // Обновление
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowBoats();
        }

        //Выбор набора
        private void listBoxStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowBoats();
        }

        //сохранение
        private void SaveToolStripMenu_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.SaveData(saveFileDialog.FileName))
                {
                    MessageBox.Show("Сохранение прошло успешно", "Результат",
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Не сохранилось", "Результат",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        //загрузка
        private void LoadToolStripMenu_Click(object sender, EventArgs args)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.LoadData(openFileDialog.FileName))
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            ReloadObjects();
        }
    }
}
