﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using ProjectBoat_bae.DrawningObjects;
using ProjectBoat_bae.Entities;
namespace ProjectBoat_bae
{
    public partial class FormBoatConfig : Form
    {

        // Переменная-выбранная лодка
        public Drawningboat? _boat = null;

        // Событие
        private event Action<Drawningboat>? EventAddBoat;

        // Добавление события
        public void AddEvent(Action<Drawningboat> ev)
        {
            if (EventAddBoat == null)
            {
                EventAddBoat = ev;
            }
            else
            {
                EventAddBoat += ev;
            }
        }

        // Конструктор
        public FormBoatConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            labelSimpleObject.MouseDown += LabelObject_MouseDown;
            labelModifiedObject.MouseDown += LabelObject_MouseDown;
            buttonCancel.Click += (s, e) => Close();
        }

        // Метод прорисовки 
        private void DrawBoat()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _boat?.SetPosition(5, 5);
            _boat?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }

        // Передаем информацию при нажатии на Label
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        // Проверка получаемой информации (ее типа на соответствие требуемому)
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        // Действия при приеме перетаскиваемой информации
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _boat = new Drawningboat((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _boat = new DrawningMotorBoat((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black,
                        checkBoxBoat.Checked, checkBoxBoat2.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            LabelColor.BackColor = Color.Empty;
            LabelAdditionalColor.BackColor = Color.Empty;
            DrawBoat();
        }

        // Отправляем цвет с панели
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor,
                DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat == null)
                return;
            switch (((Label)sender).Name)
            {
                case "LabelColor":
                    _boat?.EntityBoat?.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "LabelAdditionalColor":
                    if (!(_boat is ProjectBoat_bae))
                        return;
                    (_boat.EntityBoat as EntityMotorBoat)?.setAdditionalColor(color:
                        (Color)e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawBoat();
        }

        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelAdditionalColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)) && _boat != null && _boat is Drawningboat)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = (Color)e.Data.GetData(typeof(Color));
            //Приведение к EntityTrolleybus для замены доп. цвета
            EntityMotorBoat? _motorboat = _boat.EntityBoat as EntityMotorBoat;
            _motorboat.setAdditionalColor(color);
            DrawBoat();
        }

        private void buttonOk_Click_1(object sender, EventArgs e)
        {
            EventAddBoat?.Invoke(_boat);
            Close();
        }
    }
}
