﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using ProjectBoat_bae.DrawningObjects;
using ProjectBoat_bae.MovementStrategy;
using ProjectBoat_bae.Entities;

namespace ProjectBoat_bae.Generics
{
    internal class BoatsGenericStorage
    {
        readonly Dictionary<string, BoatsGenericCollection<Drawningboat, DrawningObjectBoat>> _boatStorages;
        public List<string> Keys => _boatStorages.Keys.ToList();

        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        public BoatsGenericStorage(int pictureWidth, int pictureHeight)
        {
            _boatStorages = new Dictionary<string, BoatsGenericCollection<Drawningboat, DrawningObjectBoat>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddSet(string name)
        {
            if (_boatStorages.ContainsKey(name)) return;
            _boatStorages[name] = new BoatsGenericCollection<Drawningboat, DrawningObjectBoat>(_pictureWidth, _pictureHeight);
        }

        public void DelSet(string name)
        {
            if (!_boatStorages.ContainsKey(name)) return;
            _boatStorages.Remove(name);
        }

        public BoatsGenericCollection<Drawningboat, DrawningObjectBoat>?
            this[string ind]
        {
            get
            {
                if (_boatStorages.ContainsKey(ind)) return _boatStorages[ind];
                return null;
            }
        }

        private static readonly char _separatorForKeyValue = '|';
        private readonly char _separatorRecords = ';';
        private static readonly char _separatorForObject = ':';

        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string, BoatsGenericCollection<Drawningboat, DrawningObjectBoat>> record in _boatStorages)
            {
                StringBuilder records = new();
                foreach (Drawningboat? elem in record.Value.GetBoats)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.Write($"PlaneStorage{Environment.NewLine}{data}");
            }
            return true;
        }

        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }

            using (StreamReader fs = File.OpenText(filename))
            {
                string str = fs.ReadLine();
                if (str == null || str.Length == 0)
                {
                    return false;
                }
                if (!str.StartsWith("BoatStorage"))
                {
                    return false;
                }

                _boatStorages.Clear();
                string strs = "";

                while ((strs = fs.ReadLine()) != null)
                {
                    if (strs == null)
                    {
                        return false;
                    }

                    string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 2)
                    {
                        continue;
                    }
                    BoatsGenericCollection<Drawningboat, DrawningObjectBoat> collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set)
                    {
                        Drawningboat? boat = elem?.CreateDrawningBoat(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (boat != null)
                        {
                            if (!(collection + boat))
                            {
                                return false;
                            }
                        }
                    }
                    _boatStorages.Add(record[0], collection);
                }
                return true;
            }
        }
    }
}
