﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBoat_bae.Generics
{
    internal class SetGeneric<T> where T : class
    {
        private readonly List<T?> _places;
        public int Count => _places.Count;

        // Максимальное количество объектов в списке
        private readonly int _maxCount;

        // Конструктор
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(_maxCount);
        }

        public bool Insert(T boat)
        {
            return Insert(boat, 0);
        }
        public bool Insert(T boat, int position)
        {
            if (position < 0 || position >= _maxCount)
                return false;

            if (Count >= _maxCount)
                return false;
            _places.Insert(0, boat);
            return true;
        }

        public bool Remove(int position)
        {
            if (position < 0 || position > _maxCount)
                return false;
            if (position >= Count)
                return false;
            _places.RemoveAt(position);
            return true;
        }
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _maxCount)
                    return;
                _places[position] = value;
            }
        }

        public IEnumerable<T?> GetBoats(int? maxBoats = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxBoats.HasValue && i == maxBoats.Value)
                {
                    yield break;
                }
            }
        }
    }

}